using System;
namespace symmetry
{
	
	/// <summary> Manages a simulation of physical forces acting on bodies. To create a
	/// custom ForceSimulator, add the desired force functions and choose an
	/// appropriate integrator.
	/// 
	/// </summary>
	/// <version>  1.0
	/// </version>
	/// <author>  <a href="http://jheer.org">Jeffrey Heer</a> prefuse(AT)jheer.org
	/// </author>
	public class ForceSimulator
	{
		virtual public float SpeedLimit
		{
			get
			{
				return speedLimit;
			}
			//
			
			set
			{
				speedLimit = value;
			}
			//
			
		}
		virtual public Force[] Forces
		{
			get
			{
				Force[] rv = new Force[iflen + sflen];
				Array.Copy((System.Array) iforces, 0, (System.Array) rv, 0, iflen);
				Array.Copy((System.Array) sforces, 0, (System.Array) rv, iflen, sflen);
				return rv;
			}
			//
			
		}
		virtual public System.Collections.IEnumerator Items
		{
			get
			{
				return items.GetEnumerator();
			}
			//
			
		}
		virtual public System.Collections.IEnumerator Springs
		{
			get
			{
				return springs.GetEnumerator();
			}
			//
			
		}
		
		private SupportClass.SetSupport items;
		private SupportClass.SetSupport springs;
		private Force[] iforces;
		private Force[] sforces;
		private int iflen, sflen;
		private Integrator integrator;
		private float speedLimit = 1.0f;
		
		public ForceSimulator():this(new RungeKuttaIntegrator())
		{
		} //
		
		public ForceSimulator(Integrator integr)
		{
			integrator = integr;
			iforces = new Force[5];
			sforces = new Force[5];
			iflen = 0;
			sflen = 0;
			items = new SupportClass.HashSetSupport();
			springs = new SupportClass.HashSetSupport();
		} //
		
		public virtual void  clear()
		{
			items.Clear();
			System.Collections.IEnumerator siter = springs.GetEnumerator();
			Spring.SpringFactory f = Spring.Factory;
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (siter.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				f.reclaim((Spring) siter.Current);
			}
			springs.Clear();
		} //
		
		public virtual void  addForce(Force f)
		{
			if (f.ItemForce)
			{
				if (iforces.Length == iflen)
				{
					// resize necessary
					Force[] newf = new Force[iflen + 10];
					Array.Copy((System.Array) iforces, 0, (System.Array) newf, 0, iforces.Length);
					iforces = newf;
				}
				iforces[iflen++] = f;
			}
			if (f.isSpringForce())
			{
				if (sforces.Length == sflen)
				{
					// resize necessary
					Force[] newf = new Force[sflen + 10];
					Array.Copy((System.Array) sforces, 0, (System.Array) newf, 0, sforces.Length);
					sforces = newf;
				}
				sforces[sflen++] = f;
			}
		} //
		
		public virtual void  addItem(ForceItem item)
		{
			
			
			items.Add(item);
		} //
		
		public virtual bool removeItem(ForceItem item)
		{
			return items.Remove(item);
		} //
		
		public virtual Spring addSpring(ForceItem item1, ForceItem item2)
		{
			return addSpring(item1, item2, - 1.0f, - 1.0f);
		} //
		
		public virtual Spring addSpring(ForceItem item1, ForceItem item2, float length)
		{
			return addSpring(item1, item2, - 1.0f, length);
		} //
		
		public virtual Spring addSpring(ForceItem item1, ForceItem item2, float coeff, float length)
		{
			if (item1 == null || item2 == null)
				throw new System.ArgumentException("ForceItems must be non-null");
			Spring s = Spring.Factory.getSpring(item1, item2, coeff, length);
			springs.Add(s);
			return s;
		} //
		
		public virtual bool removeSpring(Spring s)
		{
			return springs.Remove(s);
		}
		
		public virtual void  runSimulator(long timestep)
		{
			accumulate();
			integrator.integrate(this, timestep);
		} //
		
		/// <summary> Accumulate all forces acting on the items in this simulation</summary>
		public virtual void  accumulate()
		{
			for (int i = 0; i < iflen; i++)
				iforces[i].init(this);
			for (int i = 0; i < sflen; i++)
				sforces[i].init(this);
			System.Collections.IEnumerator itemIter = items.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (itemIter.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				ForceItem item = (ForceItem) itemIter.Current;
				item.force[0] = 0.0f; item.force[1] = 0.0f;
				for (int i = 0; i < iflen; i++)
					iforces[i].getForce(item);
			}
			System.Collections.IEnumerator springIter = springs.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (springIter.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Spring s = (Spring) springIter.Current;
				for (int i = 0; i < sflen; i++)
				{
					sforces[i].getForce(s);
				}
			}
		} //
	} // end of class ForceSimulator
}