using System;
namespace symmetry
{
	
	/// <summary> Computes the force on a node resulting from treating incident edges
	/// as a system of springs.
	/// 
	/// </summary>
	/// <version>  1.0
	/// </version>
	/// <author>  <a href="http://jheer.org">Jeffrey Heer</a> prefuse(AT)jheer.org
	/// </author>
	public class SpringForce:AbstractForce
	{
		override protected internal System.String[] ParameterNames
		{
			get
			{
				return pnames;
			}
			//
			
		}
		
		private static System.String[] pnames = new System.String[]{"SpringCoefficient", "DefaultSpringLength"};
		
		public const float DEFAULT_SPRING_COEFF = 1e-4f;
		public const float DEFAULT_SPRING_LENGTH = 100;
		public const int SPRING_COEFF = 0;
		public const int SPRING_LENGTH = 1;
		
		private ForceSimulator fsim;
		
		public SpringForce(float springCoeff, float defaultLength)
		{
			params_Renamed = new float[]{springCoeff, defaultLength};
		} //
		
		/// <summary> Constructs a new SpringForce instance.</summary>
		public SpringForce():this(DEFAULT_SPRING_COEFF, DEFAULT_SPRING_LENGTH)
		{
		} //
		
		public override bool isSpringForce()
		{
			return true;
		} //
		
		/// <summary> Initialize this force function.
		/// Subclasses should override this method with any needed initialization.
		/// </summary>
		/// <param name="fsim">the encompassing ForceSimulator
		/// </param>
		public override void  init(ForceSimulator fsim)
		{
			this.fsim = fsim;
		} //    
		
		/// <summary> Calculates the force vector acting on the items due to the given spring.</summary>
		/// <param name="s">the Spring for which to compute the force
		/// </param>
		public override void  getForce(Spring s)
		{
			ForceItem item1 = s.item1;
			ForceItem item2 = s.item2;
			float length = (s.length < 0?params_Renamed[SPRING_LENGTH]:s.length);
			float x1 = item1.location[0], y1 = item1.location[1];
			float x2 = item2.location[0], y2 = item2.location[1];
			float dx = x2 - x1, dy = y2 - y1;
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			float r = (float) System.Math.Sqrt(dx * dx + dy * dy);
			if (r == 0.0)
			{
				//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
				dx = ((float) SupportClass.Random.NextDouble() - 0.5f) / 50.0f;
				//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
				dy = ((float) SupportClass.Random.NextDouble() - 0.5f) / 50.0f;
				//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
				r = (float) System.Math.Sqrt(dx * dx + dy * dy);
			}
			float d = r - length;
			float coeff = (s.coeff < 0?params_Renamed[SPRING_COEFF]:s.coeff) * d / r;
			item1.force[0] += coeff * dx;
			item1.force[1] += coeff * dy;
			item2.force[0] += (- coeff) * dx;
			item2.force[1] += (- coeff) * dy;
		} //
	} // end of class SpringForce
}